﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace Hyperbolic.Model
{
    /// <smmary> The INode interface should be implemented by object that are node of the tree that want to be displayed in the TreeMap.  
    ///If you have already a tree structure, just implements this interface in node of the tree.
    /// </summary>
    public interface INode
    {
        /// <summary> Returns the children of this node in an Enumeration.
        ///If this object does not have children, it should return an empty Enumeration.
        ///All objects contained in the Enumeration should implements IHtNode.
        ///<returns>an Enumeration containing childs of this node</returns>
        /// </summary>    
        ObservableCollection<INode> Children { get; }

        /// <summary> Checks if this node is a leaf or not. 
        ///A node could have no children and still not be a leaf.
        /// <returns>True if this node is a leaf; false otherwise.</returns>
        /// </summary>
        bool IsLeaf { get; }

        int Level { get; }

        /// <summary> Gets the name of this node.
        ///Used to display a label in the hyperbolic tree.     
        /// </summary>
        string Name { get; }

        string Description { get; }

    }
}
