﻿/*------------------------------------------------------------------
Copyright (c) 2008 visualSingularity.net. All rights reserved.

This file is part of Hyperbolic control.

Hyperbolic control is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Hyperbolic control is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace Hyperbolic.Model
{
  /// <smmary> The INode interface should be implemented by object that are node of the tree that we want to be displayed in the TreeMap.  
  ///If you have already a tree structure, just implements this interface in node of the tree.
  /// </summary>
  public interface IBindingNode
  {
    /// <summary> Returns the children of this node in an Enumeration.
    ///If this object does not have children, it should return an empty Enumeration.
    ///All objects contained in the Enumeration should implements IHtNode.
    ///<returns>an Enumeration containing childs of this node</returns>
    /// </summary>    
    ObservableCollection<IBindingNode> Children { get; }

    /// <summary> Checks if this node is a leaf or not. 
    ///A node could have no children and still not be a leaf.
    /// <returns>True if this node is a leaf; false otherwise.</returns>
    /// </summary>
    bool IsLeaf { get; }

    int Level { get; }

    /// <summary> Gets the name of this node.
    ///Used to display a label in the hyperbolic tree.     
    /// </summary>
    string Name { get; }

    string Description { get; }

  }
}
